#ifndef __NLL_FUNCTION_H_INCLUDED__
#define __NLL_FUNCTION_H_INCLUDED__

typedef enum {
  FUNCTION_TYPE_NONE = 0,
  FUNCTION_TYPE_FARGS,
  FUNCTION_TYPE_COPY,
  FUNCTION_TYPE_TRUE,
  FUNCTION_TYPE_FALSE,
  FUNCTION_TYPE_COND,
  FUNCTION_TYPE_AT,
  FUNCTION_TYPE_FPUSH,
  FUNCTION_TYPE_FPOP,
  FUNCTION_TYPE_STACKNUM,
  FUNCTION_TYPE_FPRINT,
  FUNCTION_TYPE_SPRINTF,
  FUNCTION_TYPE_FPRINTF,
  FUNCTION_TYPE_FPRINTM,
  FUNCTION_TYPE_INPUT,
  FUNCTION_TYPE_FWAIT,
  FUNCTION_TYPE_NOSYSTEM,
  FUNCTION_TYPE_NOSYSCALL,
#ifdef NLL_STDIO_FUNCTION
  FUNCTION_TYPE_FSTDIN,
  FUNCTION_TYPE_FSTDOUT,
#endif
#ifdef NLL_SYSTEM_FUNCTION
  FUNCTION_TYPE_SYSTEM,
#endif
#ifdef NLL_SYSCALL_FUNCTION
  FUNCTION_TYPE_SYSCALL,
#endif
  FUNCTION_TYPE_FEVAL,
  FUNCTION_TYPE_ITOC,
  FUNCTION_TYPE_CTOI,
  FUNCTION_TYPE_ITOA,
  FUNCTION_TYPE_ATOI,
#ifdef NLL_FLOATING_POINT
  FUNCTION_TYPE_ITOF,
  FUNCTION_TYPE_FTOI,
#endif
  FUNCTION_TYPE_LENGTH,
  FUNCTION_TYPE_LEFT,
  FUNCTION_TYPE_RIGHT,
  FUNCTION_TYPE_MID,
  FUNCTION_TYPE_INSTR,
  FUNCTION_TYPE_STRCPY,
  FUNCTION_TYPE_SRAND,
  FUNCTION_TYPE_RAND,
  FUNCTION_TYPE_INT,
  FUNCTION_TYPE_ABS,
  FUNCTION_TYPE_SIGN,
  FUNCTION_TYPE_TIME,
  FUNCTION_TYPE_DATE,
  FUNCTION_TYPE_MEMORY,
  FUNCTION_TYPE_PEEK,
  FUNCTION_TYPE_POKE,
  FUNCTION_TYPE_READ,
  FUNCTION_TYPE_WRITE,
  FUNCTION_TYPE_MEMCPY,
#ifdef USE_MATHLIB
  FUNCTION_TYPE_PI,
  FUNCTION_TYPE_RADIAN,
  FUNCTION_TYPE_DEGREE,
  FUNCTION_TYPE_SIN,
  FUNCTION_TYPE_COS,
  FUNCTION_TYPE_TAN,
  FUNCTION_TYPE_ASIN,
  FUNCTION_TYPE_ACOS,
  FUNCTION_TYPE_ATAN,
  FUNCTION_TYPE_SINH,
  FUNCTION_TYPE_COSH,
  FUNCTION_TYPE_TANH,
  FUNCTION_TYPE_EXP,
  FUNCTION_TYPE_LOG,
  FUNCTION_TYPE_LOG10,
  FUNCTION_TYPE_SQRT,
  FUNCTION_TYPE_ATAN2,
  FUNCTION_TYPE_POW,
  FUNCTION_TYPE_FMOD,
  FUNCTION_TYPE_INFINITY,
  FUNCTION_TYPE_NAN,
  FUNCTION_TYPE_ISFINITE,
  FUNCTION_TYPE_ISINF,
  FUNCTION_TYPE_ISNAN,
  FUNCTION_TYPE_ISNORMAL,
#endif
  FUNCTION_TYPE_FOPEN,
  FUNCTION_TYPE_FCLOSE,
  FUNCTION_TYPE_FGETS,
  FUNCTION_TYPE_FPUTS,
  FUNCTION_TYPE_FREAD,
  FUNCTION_TYPE_FWRITE,
  FUNCTION_TYPE_KEYNAME,
  FUNCTION_TYPE_KEYCODE,
  FUNCTION_TYPE_SCREEN,
  FUNCTION_TYPE_SCRSTAT,
  FUNCTION_TYPE_FLUSH,
  FUNCTION_TYPE_GETCOLOR,
  FUNCTION_TYPE_SETCOLOR,
  FUNCTION_TYPE_GETATTR,
  FUNCTION_TYPE_SETATTR,
  FUNCTION_TYPE_GETPOS,
  FUNCTION_TYPE_MOVE,
  FUNCTION_TYPE_GETCH,
  FUNCTION_TYPE_CLEAR,
  FUNCTION_TYPE_SCROLL,
  FUNCTION_TYPE_LINE,
  FUNCTION_TYPE_BOX,
  FUNCTION_TYPE_INKEY,
  FUNCTION_TYPE_GSCREEN,
  FUNCTION_TYPE_GSCRSTAT,
  FUNCTION_TYPE_GCOLOR,
  FUNCTION_TYPE_GRGB,
  FUNCTION_TYPE_GSYNC,
  FUNCTION_TYPE_GFLUSH,
  FUNCTION_TYPE_GGETSIZE,
  FUNCTION_TYPE_GGETOFFSET,
  FUNCTION_TYPE_GSETOFFSET,
  FUNCTION_TYPE_GGETSCALE,
  FUNCTION_TYPE_GSETSCALE,
  FUNCTION_TYPE_GGETCOLOR,
  FUNCTION_TYPE_GSETCOLOR,
  FUNCTION_TYPE_GGETPIXEL,
  FUNCTION_TYPE_GCLEAR,
  FUNCTION_TYPE_GSCROLL,
  FUNCTION_TYPE_GDOT,
  FUNCTION_TYPE_GLINE,
  FUNCTION_TYPE_GBOX,
  FUNCTION_TYPE_GCIRCLE,
  FUNCTION_TYPE_GPAINT,
  FUNCTION_TYPE_GCOPYIMAGE,
  FUNCTION_TYPE_GINKEY,
  FUNCTION_TYPE_GKEYSTAT,
  FUNCTION_TYPE_GMOUSEPOS,
  FUNCTION_TYPE_GINMOUSE,
  FUNCTION_TYPE_GMOUSESTAT,
  FUNCTION_TYPE_GMAINIMAGE,
  FUNCTION_TYPE_GALLOCIMAGE,
  FUNCTION_TYPE_GFREEIMAGE,
  FUNCTION_TYPE_GLOADIMAGE,
  FUNCTION_TYPE_GSAVEIMAGE,
  FUNCTION_TYPE_ACLEAR,
  FUNCTION_TYPE_AOSC,
  FUNCTION_TYPE_AADDOUT,
  FUNCTION_TYPE_AENVPOINT,
  FUNCTION_TYPE_APLAY,
} function_type_t;

typedef int (*function_retnum_func_t)(value_t args);

struct function {
  const char *name;
  char *other_name;
  function_type_t type;
  int retval_number;
  function_retnum_func_t retnum_func;
  int (*function)(value_t *valuep, value_t args);
};

const char *function_name(function_t function);
int function_search(const char *name, function_t *functionp);
int function_list(FILE *fp);
int function_proc(function_t function, value_t *valuep, value_t args);

#endif
